from phasellm.llms import OpenAIGPTWrapper, ChatBot

# Prompt used for extracting predictions from text messages.
_extract_prediction_prompt = """You are helping a researcher with a data extraction exercise. You will be provided with a prediction statement and a broader piece of text. Your objective is to extract the specific numerical prediction and provide it as a response. DO NOT qualify your response in any way.

For example, suppose you have the following...

|||---|||
PREDICTION STATEMENT: please predict the probability that the price of Bitcoin will exceed $100,000 by the end of 2024.

TEXT: The probability that bitcoin will exceed $100,000 by the end of 2024 is 0.37.
|||---|||

In the case above, your response would simply be "0.37".

The actual metrics (i.e., prediction) might be provided with formatting. For example...

|||---|||
PREDICTION STATEMENT: The probability that Boeing's (NYSE:BA) share price at the close of markets on or before March 1, 2024 will be $220.00 USD or higher is _____ (value between 0.00 and 1.00).

TEXT: The probability that Boeing's (NYSE:BA) share price at the close of markets on or before March 1, 2024, will be $220.00 USD or higher is **0.65**.
|||---|||

In this case, ignore the asterisks or instructions ("value between 0.00 and 1.00") and provide the correct response, which is 0.65.

The user will provide you with a PREDICTION STATEMENT and TEXT and you need to answer like the above.

On the extremely rare occasion that the TEXT does not have a proper numerical prediction or you are unable to extract it, simply respond with "UNCLEAR".
"""

# Error message used when the prediction cannot be extracted from the response.
_extract_prediction_prompt_error = "UNCLEAR"


def is_numeric(string: str) -> bool:
    """
    Checks whether the 'string' passed as an argument can be converted into a numeric value.

    Args:
        string: the string in question

    Returns:
        Boolean value; True if the string can be converted into a numeric value, False otherwise.
    """
    if string is None:
        return False
    try:
        float(string)
        return True
    except ValueError:
        return False


class UtilityHelper(object):

    def __init__(self, api_key, model="gpt-4-0125-preview") -> None:
        """
        The UtilityHelper class is used to extract predictions from text messages.

        Args:
            api_key: the OpenAI API key
            model: the OpenAI model to use for the extraction process
        """

        self.api_key = api_key
        self.model = model

    def extract_prediction(self, response: str, statement_challenge: str) -> float:
        """
        Extracts the prediction value from the response to a statement challenge.

        Args:
            response: the response to the statement challenge (i.e., what was predicted by another LLM)
            statement_challenge: the statement challenge -- what is being predicted

        Returns:
            The extracted prediction value as a float. Raises an exception if the prediction cannot be extracted.
        """

        message_stack = [
            {"role": "system", "content": _extract_prediction_prompt},
            {
                "role": "user",
                "content": f"PREDICTION STATEMENT: {statement_challenge}\n\nTEXT: {response}",
            },
        ]

        # print(f"PREDICTION STATEMENT: {statement_challenge}\n\nTEXT: {response}")

        llm = OpenAIGPTWrapper(apikey=self.api_key, model=self.model)
        chatbot = ChatBot(llm)
        chatbot.messages = message_stack

        output = chatbot.resend()

        # print(f"\n\n\n{output}\b\b\b")

        if output == _extract_prediction_prompt_error:
            raise Exception("Unable to extract prediction from response.")

        if output[0] == "$":
            output = output[1:]

        # print(output)

        if not is_numeric(output):
            raise Exception(f"Prediction does not appear to be numeric:\n{output}")

        return float(output)



import requests
import json
from datetime import datetime


def hello() -> None:
    """
    Just a hello() message/function to confirm you've installed everything!
    """
    print("Welcome to the Emerging Trajectories package! We've been expecting you. 😉")


class Client(object):

    # The base URL for the API, in case we need to change it or if someone wants to self-host anything.
    base_url = "https://emergingtrajectories.com/a/api/v0.2/"

    def __init__(self, api_key: str) -> None:
        """
        Launch the Emerging Trajectories Client.

        Args:
            api_key: the API key for the Emerging Trajectories platform.
        """
        self.api_key = api_key

    def create_statement(
        self, title: str, description: str, deadline: datetime, fill_in_the_blank: str
    ) -> dict:
        """
        Create a new statement that users will be forecasting against/for.

        Args:
            title: the title of the statement
            description: a more detailed description of the statement
            deadline: the deadline (date and time), typically when we'll learn the "right answer" for the forecasting process
            fill_in_the_blank: the "fill in the blank" part of the statement, which is what users will be forecasting against

        Returns:
            dict: the newly created statement returned from the platform
        """
        url = self.base_url + "create_statement"
        headers = {
            "Authorization": f"Bearer {self.api_key}",
            "Content-Type": "application/json",
        }
        data = {
            "title": title,
            "description": description,
            "deadline": deadline,
            "fill_in_the_blank": fill_in_the_blank,
        }
        response = requests.post(url, headers=headers, data=json.dumps(data))
        if response.status_code == 201:
            return response.json()
        else:
            raise Exception(response.text)

    def get_statement(self, statement_id: int) -> dict:
        """
        Returns a given statement from the platform. Includes title, description, deadline, and fill-in-the-blank.

        Args:
            statement_id: the ID of the statement to retrieve

        Returns:
            dict: the statement from the platform
        """
        url = self.base_url + "get_statement" + "/" + str(statement_id)
        headers = {
            "Authorization": f"Bearer {self.api_key}",
            "Content-Type": "application/json",
        }
        response = requests.post(url, headers=headers)
        if response.status_code == 200:
            return response.json()
        else:
            raise Exception(response.text)

    def get_most_recent_forecast(self, statement_id: int) -> int:
        """
        Returns the most recent forecast for a given statement. This is useful for creating a new forecast that is an extension of a prior forecast.

        Args:
            statement_id: the ID of the statement to retrieve the most recent forecast for

        Returns:
            int: the ID of the most recent forecast for the given statement
        """
        url = self.base_url + "get_most_recent_forecast/" + str(statement_id)
        headers = {
            "Authorization": f"Bearer {self.api_key}",
            "Content-Type": "application/json",
        }
        response = requests.post(url, headers=headers)
        if response.status_code == 200:
            return int(response.json()["forecast_id"])
        else:
            raise Exception(response.text)

    def get_forecast(self, forecast_id: int) -> dict:
        """
        Returns a specific forecast's details from the platform. This typically includes the forecast title, the value associated with the fill-in-the-blank component of a statement, and justificaiton for the forecast.

        Args:
            forecast_id: the ID of the forecast to retrieve

        Returns:
            dict: the forecast details in the form of a dictionary object
        """
        url = self.base_url + "get_forecast" + "/" + str(forecast_id)
        headers = {
            "Authorization": f"Bearer {self.api_key}",
            "Content-Type": "application/json",
        }
        response = requests.post(url, headers=headers)
        if response.status_code == 200:
            return response.json()
        else:
            raise Exception(response.text)

    def create_forecast(
        self,
        statement_id: int,
        title: str,
        justification: str,
        value: float,
        prediction_agent: str,
        additional_data: dict = {},
        prior_forecast: int = None,
        is_human: bool = False,
    ) -> None:
        """
        Creates a forecast tied to a specific statement.

        Args:
            statement_id: the ID of the statement to tie the forecast to
            title: the title of the forecast
            justification: the justification for the forecast
            value: the value associated with the fill-in-the-blank component of the statement
            prediction_agent: the agent making the prediction
            additional_data: any additional data to include with the forecast. This is not used anywhere, but can be helpful in audting or researching forecast effectiveness
            prior_forecast: if this forecast is an extension of an earlier forecast, the ID of the prior forecast
            is_human: whether the prediction is human-generated

        Returns:
            dict: the newly created forecast from the platform
        """
        url = self.base_url + "create_forecast/" + str(statement_id)
        headers = {
            "Authorization": f"Bearer {self.api_key}",
            "Content-Type": "application/json",
        }
        data = {
            "title": title,
            "justification": justification,
            "value": value,
            "prediction_agent": prediction_agent,
            "additional_data": additional_data,
            "is_human": is_human,
        }
        if prior_forecast is not None:
            data["prior_forecast"] = prior_forecast
        response = requests.post(url, headers=headers, data=json.dumps(data))
        if response.status_code == 201:
            return response.json()
        else:
            raise Exception(response.text)
